@@ -20,53 +20,55 @@ in the program's Context.
 Additionally, this module allows you to specify environment variables that,
 if present, will be incorporated into the stored configuration.
 -}
-module Core.Program.Arguments (
-    -- * Setup
-    Config,
-    blankConfig,
-    simpleConfig,
-    complexConfig,
-    baselineOptions,
-    Parameters (..),
-    ParameterValue (..),
+module Core.Program.Arguments
+    ( -- * Setup
+      Config
+    , blankConfig
+    , simpleConfig
+    , simpleConfig'
+    , complexConfig
+    , complexConfig'
+    , baselineOptions
+    , Parameters (..)
+    , ParameterValue (..)
 
-    -- * Options and Arguments
-    LongName (..),
-    ShortName,
-    Description,
-    Options (..),
+      -- * Options and Arguments
+    , LongName (..)
+    , ShortName
+    , Description
+    , Options (..)
 
-    -- * Programs with Commands
-    Commands (..),
-    appendOption,
+      -- * Programs with Commands
+    , Commands (..)
+    , appendOption
 
-    -- * Internals
-    parseCommandLine,
-    extractValidEnvironments,
-    InvalidCommandLine (..),
-    buildUsage,
-    buildVersion,
-    emptyParameters,
-) where
+      -- * Internals
+    , parseCommandLine
+    , extractValidEnvironments
+    , InvalidCommandLine (..)
+    , buildUsage
+    , buildVersion
+    , emptyParameters
+    ) where
 
 import Data.Hashable (Hashable)
 import Data.List qualified as List
 import Data.Maybe (fromMaybe)
 import Data.String (IsString (..))
-import Prettyprinter (
-    Doc,
-    Pretty (..),
-    align,
-    emptyDoc,
-    fillBreak,
-    fillCat,
-    fillSep,
-    hardline,
-    indent,
-    nest,
-    softline,
-    (<+>),
- )
+import Prettyprinter
+    ( Doc
+    , Pretty (..)
+    , align
+    , emptyDoc
+    , fillBreak
+    , fillCat
+    , fillSep
+    , hardline
+    , indent
+    , nest
+    , softline
+    , (<+>)
+    )
 import Prettyprinter.Util (reflow)
 import System.Environment (getProgName)
 
@@ -115,8 +117,8 @@ a @Config@ with 'simpleConfig' or 'complexConfig', and pass it to
 -}
 data Config
     = Blank
-    | Simple [Options]
-    | Complex [Commands]
+    | Simple Description [Options]
+    | Complex Description [Commands]
 
 --
 -- Those constructors are not exposed [and functions wrapping them are] partly
@@ -200,7 +202,27 @@ see 'quote' in "Core.Text.Utilities".
 @since 0.2.9
 -}
 simpleConfig :: [Options] -> Config
-simpleConfig options = Simple (options ++ baselineOptions)
+simpleConfig options = Simple emptyRope (options ++ baselineOptions)
+
+{- |
+Declare a simple configuration as with 'simpleConfig', along with a
+descriptive precis that will be printed at the top of the @--help@ output.
+
+
+@
+\$ __./snippet --help__
+A small but very useful program.
+
+Usage:
+
+    snippet [OPTIONS] <filename>
+...
+@
+
+@since 0.6.5
+-}
+simpleConfig' :: Description -> [Options] -> Config
+simpleConfig' description options = Simple description (options ++ baselineOptions)
 
 {- |
 Declare a complex configuration (implying a larger tool with various
@@ -286,7 +308,20 @@ see 'quote' in "Core.Text.Utilities".
 @since 0.2.9
 -}
 complexConfig :: [Commands] -> Config
-complexConfig commands = Complex (commands ++ [Global baselineOptions])
+complexConfig commands = Complex emptyRope (commands ++ [Global baselineOptions])
+
+{- |
+Declare a complex configuration as with 'complexConfig', along with a
+descriptive precis that will be printed at the top of the @--help@ output when
+requesting help for the program as a whole.
+
+If help is requested for one of the sub commands, the description from the
+'Command' constructor will be used at the top of the output.
+
+@since 0.6.5
+-}
+complexConfig' :: Description -> [Commands] -> Config
+complexConfig' precis commands = Complex precis (commands ++ [Global baselineOptions])
 
 {- |
 Description of the command-line structure of a program which has
@@ -332,7 +367,7 @@ description you supply alongside.
 
 @
         [ ...
-        , 'Remaining' "The files you wish to delete permanently."
+        , 'Remaining' \"The files you wish to delete permanently.\"
         , ...
         ]
 @
@@ -344,7 +379,17 @@ with an underscore:
 
 @
         [ ...
-        , 'Variable' \"CRAZY_MODE\" "Specify how many crazies to activate."
+        , 'Variable' \"CRAZY_MODE\" \"Specify how many crazies to activate.\"
+        , ...
+        ]
+@
+
+Finally, there is a 'Descriptive' constructor which allows you to put a piece
+of header text as a descriptive summary in help output before it starts
+enumerating the options and arguments.
+
+@
+        [ 'Descriptive' \"A program to evaluate just how crazy you are.\"
         , ...
         ]
 @
@@ -360,8 +405,8 @@ appendOption :: Options -> Config -> Config
 appendOption option config =
     case config of
         Blank -> Blank
-        Simple options -> Simple (options ++ [option])
-        Complex commands -> Complex (commands ++ [Global [option]])
+        Simple precis options -> Simple precis (options ++ [option])
+        Complex precis commands -> Complex precis (commands ++ [Global [option]])
 
 {- |
 Individual parameters read in off the command-line can either have a value
@@ -498,12 +543,12 @@ with complex values escaped according to the rules of your shell:
 
 For options valid in this program, please see --help.
         |]
-             in one ++ two
+            in  one ++ two
         UnknownOption name -> "Sorry, option '" ++ name ++ "' not recognized."
         MissingArgument (LongName name) -> "Mandatory argument '" ++ name ++ "' missing."
         UnexpectedArguments args ->
             let quoted = List.intercalate "', '" args
-             in [quote|
+            in  [quote|
 Unexpected trailing arguments:
 
 |]
@@ -550,14 +595,14 @@ initializing).
 parseCommandLine :: Config -> [String] -> Either InvalidCommandLine Parameters
 parseCommandLine config argv = case config of
     Blank -> return (Parameters Nothing emptyMap [] emptyMap)
-    Simple options -> do
+    Simple _ options -> do
         (params, remainder) <- extractor Nothing options argv
         checkRemainder options remainder
         return (Parameters Nothing params remainder emptyMap)
-    Complex commands ->
+    Complex _ commands ->
         let globalOptions = extractGlobalOptions commands
             modes = extractValidModes commands
-         in do
+        in  do
                 (possibles, argv') <- splitCommandLine1 argv
                 (params1, _) <- extractor Nothing globalOptions possibles
                 (first, moreArgs) <- splitCommandLine2 argv'
@@ -572,7 +617,7 @@ parseCommandLine config argv = case config of
             valids = extractValidNames options
             shorts = extractShortNames options
             needed = extractRequiredArguments options
-         in do
+        in  do
                 list1 <- parsePossibleOptions mode valids shorts possibles
                 (list2, arguments') <- parseRequiredArguments needed arguments
                 pure (((<>) (intoMap list1) (intoMap list2)), arguments')
@@ -602,12 +647,12 @@ isOption arg = case arg of
     ('-' : _) -> True
     _ -> False
 
-parsePossibleOptions ::
-    Maybe LongName ->
-    Set LongName ->
-    Map ShortName LongName ->
-    [String] ->
-    Either InvalidCommandLine [(LongName, ParameterValue)]
+parsePossibleOptions
+    :: Maybe LongName
+    -> Set LongName
+    -> Map ShortName LongName
+    -> [String]
+    -> Either InvalidCommandLine [(LongName, ParameterValue)]
 parsePossibleOptions mode valids shorts args = mapM f args
   where
     f arg = case arg of
@@ -626,7 +671,7 @@ parsePossibleOptions mode valids shorts args = mapM f args
             value' = case List.uncons value of
                 Just (_, remainder) -> Value remainder
                 Nothing -> Empty
-         in if containsElement candidate valids
+        in  if containsElement candidate valids
                 then Right (candidate, value')
                 else Left (UnknownOption ("--" ++ name))
 
@@ -636,10 +681,10 @@ parsePossibleOptions mode valids shorts args = mapM f args
             Just name -> Right (name, Empty)
             Nothing -> Left (UnknownOption ['-', c])
 
-parseRequiredArguments ::
-    [LongName] ->
-    [String] ->
-    Either InvalidCommandLine ([(LongName, ParameterValue)], [String])
+parseRequiredArguments
+    :: [LongName]
+    -> [String]
+    -> Either InvalidCommandLine ([(LongName, ParameterValue)], [String])
 parseRequiredArguments needed argv = iter needed argv
   where
     iter :: [LongName] -> [String] -> Either InvalidCommandLine ([(LongName, ParameterValue)], [String])
@@ -650,18 +695,18 @@ parseRequiredArguments needed argv = iter needed argv
     iter (name : _) [] = Left (MissingArgument name)
     iter (name : names) (arg : args) =
         let deeper = iter names args
-         in case deeper of
+        in  case deeper of
                 Left e -> Left e
                 Right (list, remainder) -> Right (((name, Value arg) : list), remainder)
 
-parseIndicatedCommand ::
-    Map LongName [Options] ->
-    String ->
-    Either InvalidCommandLine (LongName, [Options])
+parseIndicatedCommand
+    :: Map LongName (Description, [Options])
+    -> String
+    -> Either InvalidCommandLine (LongName, [Options])
 parseIndicatedCommand modes first =
     let candidate = LongName first
-     in case lookupKeyValue candidate modes of
-            Just options -> Right (candidate, options)
+    in  case lookupKeyValue candidate modes of
+            Just (_, options) -> Right (candidate, options)
             Nothing -> Left (UnknownCommand first)
 
 --
@@ -702,12 +747,12 @@ extractGlobalOptions commands =
     j (Global options) valids = options ++ valids
     j _ valids = valids
 
-extractValidModes :: [Commands] -> Map LongName [Options]
+extractValidModes :: [Commands] -> Map LongName (Description, [Options])
 extractValidModes commands =
     List.foldl' k emptyMap commands
   where
-    k :: Map LongName [Options] -> Commands -> Map LongName [Options]
-    k modes (Command longname _ options) = insertKeyValue longname options modes
+    k :: Map LongName (Description, [Options]) -> Commands -> Map LongName (Description, [Options])
+    k modes (Command longname description options) = insertKeyValue longname (description, options) modes
     k modes _ = modes
 
 {-
@@ -722,14 +767,14 @@ We do it this way so that `parseCommandLine` can pas the global options to
 splitCommandLine1 :: [String] -> Either InvalidCommandLine ([String], [String])
 splitCommandLine1 args =
     let (possibles, remainder) = List.span isOption args
-     in if null possibles && null remainder
+    in  if null possibles && null remainder
             then Left NoCommandFound
             else Right (possibles, remainder)
 
 splitCommandLine2 :: [String] -> Either InvalidCommandLine (String, [String])
 splitCommandLine2 argv' =
     let x = List.uncons argv'
-     in case x of
+    in  case x of
             Just (mode, remainingArgs) -> Right (mode, remainingArgs)
             Nothing -> Left NoCommandFound
 
@@ -740,14 +785,14 @@ splitCommandLine2 argv' =
 extractValidEnvironments :: Maybe LongName -> Config -> Set LongName
 extractValidEnvironments mode config = case config of
     Blank -> emptySet
-    Simple options -> extractVariableNames options
-    Complex commands ->
+    Simple _ options -> extractVariableNames options
+    Complex _ commands ->
         let globals = extractGlobalOptions commands
             variables1 = extractVariableNames globals
 
             locals = extractLocalVariables commands (fromMaybe "" mode)
             variables2 = extractVariableNames locals
-         in variables1 <> variables2
+        in  variables1 <> variables2
 
 extractLocalVariables :: [Commands] -> LongName -> [Options]
 extractLocalVariables commands mode =
@@ -775,9 +820,12 @@ extractVariableNames options =
 buildUsage :: Config -> Maybe LongName -> Doc ann
 buildUsage config mode = case config of
     Blank -> emptyDoc
-    Simple options ->
+    Simple precis options ->
         let (o, a, v) = partitionParameters options
-         in "Usage:" <> hardline <> hardline
+        in  formatPrecis precis
+                <> "Usage:"
+                <> hardline
+                <> hardline
                 <> indent
                     4
                     ( nest
@@ -797,24 +845,28 @@ buildUsage config mode = case config of
                 <> formatParameters a
                 <> variablesHeading v
                 <> formatParameters v
-    Complex commands ->
+    Complex precis commands ->
         let globalOptions = extractGlobalOptions commands
             modes = extractValidModes commands
 
             (oG, _, vG) = partitionParameters globalOptions
-         in "Usage:" <> hardline <> hardline <> case mode of
+        in  case mode of
                 Nothing ->
-                    indent
-                        2
-                        ( nest
-                            4
-                            ( fillCat
-                                [ pretty programName
-                                , globalSummary oG
-                                , commandSummary modes
-                                ]
+                    formatPrecis precis
+                        <> "Usage:"
+                        <> hardline
+                        <> hardline
+                        <> indent
+                            2
+                            ( nest
+                                4
+                                ( fillCat
+                                    [ pretty programName
+                                    , globalSummary oG
+                                    , commandSummary modes
+                                    ]
+                                )
                             )
-                        )
                         <> hardline
                         <> globalHeading oG
                         <> formatParameters oG
@@ -823,23 +875,27 @@ buildUsage config mode = case config of
                         <> variablesHeading vG
                         <> formatParameters vG
                 Just longname ->
-                    let (oL, aL, vL) = case lookupKeyValue longname modes of
-                            Just localOptions -> partitionParameters localOptions
+                    let (dL, (oL, aL, vL)) = case lookupKeyValue longname modes of
+                            Just (description, localOptions) -> (description, partitionParameters localOptions)
                             Nothing -> error "Illegal State"
-                     in indent
-                            2
-                            ( nest
-                                4
-                                ( fillCat
-                                    [ pretty programName
-                                    , globalSummary oG
-                                    , commandSummary modes
-                                    , localSummary oL
-                                    , argumentsSummary aL
-                                    , remainingSummary aL
-                                    ]
+                    in  formatPrecis dL
+                            <> "Usage:"
+                            <> hardline
+                            <> hardline
+                            <> indent
+                                2
+                                ( nest
+                                    4
+                                    ( fillCat
+                                        [ pretty programName
+                                        , globalSummary oG
+                                        , commandSummary modes
+                                        , localSummary oL
+                                        , argumentsSummary aL
+                                        , remainingSummary aL
+                                        ]
+                                    )
                                 )
-                            )
                             <> hardline
                             <> localHeading oL
                             <> formatParameters oL
@@ -848,6 +904,11 @@ buildUsage config mode = case config of
                             <> variablesHeading vL
                             <> formatParameters vL
   where
+    formatPrecis :: Description -> Doc ann
+    formatPrecis precis = case widthRope precis of
+        0 -> emptyDoc
+        _ -> reflow (fromRope precis) <> hardline <> hardline
+
     partitionParameters :: [Options] -> ([Options], [Options], [Options])
     partitionParameters options = List.foldl' f ([], [], []) options
 
@@ -913,7 +974,7 @@ buildUsage config mode = case config of
                 Nothing -> "      --"
             l = pretty longname
             d = fromRope description
-         in case valued of
+        in  case valued of
                 Empty ->
                     fillBreak 16 (s <> l <> " ") <+> align (reflow d) <> hardline <> acc
                 Value label ->
@@ -921,14 +982,14 @@ buildUsage config mode = case config of
     g acc (Argument longname description) =
         let l = pretty longname
             d = fromRope description
-         in fillBreak 16 ("  <" <> l <> "> ") <+> align (reflow d) <> hardline <> acc
+        in  fillBreak 16 ("  <" <> l <> "> ") <+> align (reflow d) <> hardline <> acc
     g acc (Remaining description) =
         let d = fromRope description
-         in fillBreak 16 ("  " <> "... ") <+> align (reflow d) <> hardline <> acc
+        in  fillBreak 16 ("  " <> "... ") <+> align (reflow d) <> hardline <> acc
     g acc (Variable longname description) =
         let l = pretty longname
             d = fromRope description
-         in fillBreak 16 ("  " <> l <> " ") <+> align (reflow d) <> hardline <> acc
+        in  fillBreak 16 ("  " <> l <> " ") <+> align (reflow d) <> hardline <> acc
 
     formatCommands :: [Commands] -> Doc ann
     formatCommands commands = hardline <> List.foldl' h emptyDoc commands
@@ -937,12 +998,12 @@ buildUsage config mode = case config of
     h acc (Command longname description _) =
         let l = pretty longname
             d = fromRope description
-         in acc <> fillBreak 16 ("  " <> l <> " ") <+> align (reflow d) <> hardline
+        in  acc <> fillBreak 16 ("  " <> l <> " ") <+> align (reflow d) <> hardline
     h acc _ = acc
 
 buildVersion :: Version -> Doc ann
 buildVersion version =
     pretty (projectNameFrom version)
         <+> "v"
-        <> pretty (versionNumberFrom version)
-        <> hardline
+            <> pretty (versionNumberFrom version)
+            <> hardline
