@@ -13,29 +13,29 @@
 {-# OPTIONS_HADDOCK hide #-}
 
 -- This is an Internal module, hidden from Haddock
-module Core.Program.Context (
-    Datum (..),
-    emptyDatum,
-    Trace (..),
-    unTrace,
-    Span (..),
-    unSpan,
-    Context (..),
-    handleCommandLine,
-    handleVerbosityLevel,
-    handleTelemetryChoice,
-    Exporter (..),
-    Forwarder (..),
-    None (..),
-    isNone,
-    configure,
-    Verbosity (..),
-    Program (..),
-    unProgram,
-    getContext,
-    fmapContext,
-    subProgram,
-) where
+module Core.Program.Context
+    ( Datum (..)
+    , emptyDatum
+    , Trace (..)
+    , unTrace
+    , Span (..)
+    , unSpan
+    , Context (..)
+    , handleCommandLine
+    , handleVerbosityLevel
+    , handleTelemetryChoice
+    , Exporter (..)
+    , Forwarder (..)
+    , None (..)
+    , isNone
+    , configure
+    , Verbosity (..)
+    , Program (..)
+    , unProgram
+    , getContext
+    , fmapContext
+    , subProgram
+    ) where
 
 import Control.Concurrent (ThreadId)
 import Control.Concurrent.MVar (MVar, newEmptyMVar, newMVar, putMVar, readMVar)
@@ -43,6 +43,7 @@ import Control.Concurrent.STM.TQueue (TQueue, newTQueueIO)
 import Control.Concurrent.STM.TVar (TVar, newTVarIO)
 import Control.Exception.Safe qualified as Safe (throw)
 import Control.Monad.Catch (MonadCatch, MonadMask, MonadThrow (throwM))
+import Control.Monad.IO.Unlift (MonadUnliftIO (withRunInIO))
 import Control.Monad.Reader.Class (MonadReader (..))
 import Control.Monad.Trans.Reader (ReaderT (..))
 import Core.Data.Clock
@@ -174,7 +175,9 @@ data Context τ = Context
     , exitSemaphoreFrom :: MVar ExitCode
     , startTimeFrom :: MVar Time
     , verbosityLevelFrom :: MVar Verbosity
+    , outputSemaphoreFrom :: MVar ()
     , outputChannelFrom :: TQueue (Maybe Rope) -- communication channels
+    , telemetrySemaphoreFrom :: MVar ()
     , telemetryChannelFrom :: TQueue (Maybe Datum) -- machinery for telemetry
     , telemetryForwarderFrom :: Maybe Forwarder
     , currentScopeFrom :: TVar (Set ThreadId)
@@ -198,7 +201,7 @@ fmapContext f context = do
     state <- readMVar (applicationDataFrom context)
     let state' = f state
     u <- newMVar state'
-    return (context{applicationDataFrom = u})
+    return (context {applicationDataFrom = u})
 
 {- |
 A 'Program' with no user-supplied state to be threaded throughout the
@@ -309,7 +312,8 @@ reason to use this; to access your top-level application data @τ@ within the
 getContext :: Program τ (Context τ)
 getContext = do
     context <- ask
-    return context
+    pure context
+{-# INLINABLE getContext #-}
 
 {- |
 Run a subprogram from within a lifted @IO@ block.
@@ -318,6 +322,18 @@ subProgram :: Context τ -> Program τ α -> IO α
 subProgram context (Program r) = do
     runReaderT r context
 
+--
+-- This isn't needed by our packages, but it's a useful instance. This is a
+-- copy of what is in Core.Program.Unlift.withContext. I would have put this
+-- there, but it leaves an orphan.
+--
+instance MonadUnliftIO (Program τ) where
+    {-# INLINE withRunInIO #-}
+    withRunInIO action = do
+        context <- getContext
+        liftIO $ do
+            action (subProgram context)
+
 {-
 This is complicated. The **safe-exceptions** library exports a `throwM` which
 is not the `throwM` class method from MonadThrow. See
@@ -356,6 +372,8 @@ configure version t config = do
     columns <- getConsoleWidth
     coloured <- getConsoleColoured
     level <- newEmptyMVar
+    vo <- newEmptyMVar
+    vl <- newEmptyMVar
     out <- newTQueueIO
     tel <- newTQueueIO
 
@@ -375,7 +393,9 @@ configure version t config = do
             , exitSemaphoreFrom = q
             , startTimeFrom = i
             , verbosityLevelFrom = level -- will be filled in handleVerbosityLevel
+            , outputSemaphoreFrom = vo
             , outputChannelFrom = out
+            , telemetrySemaphoreFrom = vl
             , telemetryChannelFrom = tel
             , telemetryForwarderFrom = Nothing
             , currentScopeFrom = scope
@@ -487,7 +507,7 @@ queryVerbosityLevel :: Parameters -> Either ExitCode Verbosity
 queryVerbosityLevel params =
     let debug = lookupKeyValue "debug" (parameterValuesFrom params)
         verbose = lookupKeyValue "verbose" (parameterValuesFrom params)
-     in case debug of
+    in  case debug of
             Just value -> case value of
                 Empty -> Right Debug
                 Value "internal" -> Right Internal
