@@ -49,98 +49,106 @@ runtime, but to specify the allowed command-line options and expected
 arguments you can initialize your program using 'configure' and then run
 with 'executeWith'.
 -}
-module Core.Program.Execute (
-    Program (),
+module Core.Program.Execute
+    ( Program ()
 
-    -- * Running programs
-    configure,
-    execute,
-    executeWith,
+      -- * Running programs
+    , configure
+    , execute
+    , executeWith
 
-    -- * Exiting a program
-    terminate,
+      -- * Exiting a program
+    , terminate
 
-    -- * Accessing program context
-    getCommandLine,
-    queryCommandName,
-    queryOptionFlag,
-    queryOptionValue,
-    queryOptionValue',
-    queryArgument,
-    queryRemaining,
-    queryEnvironmentValue,
-    getProgramName,
-    setProgramName,
-    getVerbosityLevel,
-    setVerbosityLevel,
-    getConsoleWidth,
-    getApplicationState,
-    setApplicationState,
+      -- * Accessing program context
+    , getCommandLine
+    , queryCommandName
+    , queryOptionFlag
+    , queryOptionValue
+    , queryOptionValue'
+    , queryArgument
+    , queryRemaining
+    , queryEnvironmentValue
+    , queryEnvironmentValue'
+    , getProgramName
+    , setProgramName
+    , getVerbosityLevel
+    , setVerbosityLevel
+    , getConsoleWidth
+    , getApplicationState
+    , setApplicationState
+    , changeProgram
 
-    -- * Useful actions
-    outputEntire,
-    inputEntire,
-    execProcess,
-    sleepThread,
-    resetTimer,
-    trap_,
+      -- * Useful actions
+    , outputEntire
+    , inputEntire
+    , sleepThread
+    , resetTimer
+    , trap_
 
-    -- * Exception handling
-    catch,
-    throw,
-    try,
+      -- * Exception handling
+    , catch
+    , throw
+    , try
 
-    -- * Internals
-    Context,
-    None (..),
-    isNone,
-    unProgram,
-    invalid,
-    Boom (..),
-    loopForever,
-    lookupOptionFlag,
-    lookupOptionValue,
-    lookupArgument,
-    lookupEnvironmentValue,
-) where
+      -- * Running processes
+    , readProcess
+    , execProcess_
 
-import Control.Concurrent (
-    forkFinally,
-    forkIO,
-    killThread,
-    myThreadId,
-    threadDelay,
- )
-import Control.Concurrent.MVar (
-    MVar,
-    modifyMVar_,
-    newEmptyMVar,
-    putMVar,
-    readMVar,
-    tryPutMVar,
- )
+      -- * Internals
+    , Context
+    , None (..)
+    , isNone
+    , unProgram
+    , invalid
+    , Boom (..)
+    , loopForever
+    , lookupOptionFlag
+    , lookupOptionValue
+    , lookupArgument
+    , lookupEnvironmentValue
+    , execProcess
+    )
+where
+
+import Control.Concurrent
+    ( forkFinally
+    , forkIO
+    , killThread
+    , myThreadId
+    , threadDelay
+    )
+import Control.Concurrent.MVar
+    ( MVar
+    , modifyMVar_
+    , newMVar
+    , putMVar
+    , readMVar
+    , tryPutMVar
+    )
 import Control.Concurrent.STM (atomically)
-import Control.Concurrent.STM.TQueue (
-    TQueue,
-    readTQueue,
-    tryReadTQueue,
-    unGetTQueue,
-    writeTQueue,
- )
-import Control.Concurrent.STM.TVar (
-    readTVarIO,
- )
+import Control.Concurrent.STM.TQueue
+    ( TQueue
+    , readTQueue
+    , tryReadTQueue
+    , unGetTQueue
+    , writeTQueue
+    )
+import Control.Concurrent.STM.TVar
+    ( readTVarIO
+    )
 import Control.Exception qualified as Base (throwIO)
-import Control.Exception.Safe qualified as Safe (
-    catch,
-    throw,
- )
-import Control.Monad (
-    forM_,
-    forever,
-    void,
-    when,
- )
+import Control.Exception.Safe qualified as Safe
+    ( catch
+    , throw
+    )
+import Control.Monad
+    ( forM_
+    , forever
+    , unless
+    , void
+    , when
+    )
 import Control.Monad.Reader.Class (MonadReader (ask))
 import Core.Data.Clock
 import Core.Data.Structures
@@ -150,15 +158,15 @@ import Core.Program.Context
 import Core.Program.Exceptions
 import Core.Program.Logging
 import Core.Program.Signal
-import Core.System.Base (
-    Exception,
-    Handle,
-    SomeException,
-    displayException,
-    hFlush,
-    liftIO,
-    stdout,
- )
+import Core.System.Base
+    ( Exception
+    , Handle
+    , SomeException
+    , displayException
+    , hFlush
+    , liftIO
+    , stdout
+    )
 import Core.Text.Bytes
 import Core.Text.Rope
 import Data.ByteString qualified as B (hPut)
@@ -166,11 +174,13 @@ import Data.ByteString.Char8 qualified as C (singleton)
 import Data.List qualified as List (intersperse)
 import GHC.Conc (getNumProcessors, numCapabilities, setNumCapabilities)
 import GHC.IO.Encoding (setLocaleEncoding, utf8)
-import System.Directory (
-    findExecutable,
- )
+import System.Directory
+    ( findExecutable
+    )
 import System.Exit (ExitCode (..))
-import System.Process.Typed (nullStream, proc, readProcess, setStdin)
+import System.Posix.Internals (hostIsThreaded)
+import System.Posix.Process qualified as Posix (executeFile, exitImmediately)
+import System.Process.Typed qualified as Typed (nullStream, proc, readProcess, setStdin)
 import Prelude hiding (log)
 
 {- |
@@ -203,7 +213,7 @@ trap_ action =
         (void action)
         ( \(e :: SomeException) ->
             let text = intoRope (displayException e)
-             in do
+            in  do
                     warn "Trapped uncaught exception"
                     debug "e" text
         )
@@ -228,8 +238,14 @@ executeWith = executeActual
 
 executeActual :: Context τ -> Program τ α -> IO ()
 executeActual context0 program = do
+    -- ensure threaded runtime is active
+    unless hostIsThreaded $ do
+        putStrLn "error: Application must be compiled with -threaded GHC option"
+        Posix.exitImmediately (ExitFailure 98)
+
     -- command line +RTS -Nn -RTS value
-    when (numCapabilities == 1) (getNumProcessors >>= setNumCapabilities)
+    when (numCapabilities == 1) $ do
+        getNumProcessors >>= setNumCapabilities
 
     -- force UTF-8 working around bad VMs
     setLocaleEncoding utf8
@@ -240,23 +256,23 @@ executeActual context0 program = do
     level <- handleVerbosityLevel context
 
     let quit = exitSemaphoreFrom context
-        out = outputChannelFrom context
-        tel = telemetryChannelFrom context
-        forwarder = telemetryForwarderFrom context
+    let vo = outputSemaphoreFrom context
+    let out = outputChannelFrom context
+    let vl = telemetrySemaphoreFrom context
+    let tel = telemetryChannelFrom context
+    let forwarder = telemetryForwarderFrom context
 
     -- set up signal handlers
     _ <- forkIO $ do
         setupSignalHandlers quit level
 
     -- set up standard output
-    vo <- newEmptyMVar
     _ <-
         forkFinally
             (processStandardOutput out)
             (\_ -> putMVar vo ())
 
     -- set up debug logger
-    vl <- newEmptyMVar
     _ <-
         forkFinally
             (processTelemetryMessages forwarder level out tel)
@@ -571,6 +587,57 @@ setApplicationState user = do
         let v = applicationDataFrom context
         modifyMVar_ v (\_ -> pure user)
 
+{- |
+Sometimes you need to change the type of the application state from what is
+present at the top-level when the program starts.
+
+While the original intent of providing an initial value of type @τ@ to
+'configure' was that your application state would be available at startup, an
+alternative pattern is to form the application state as the first actions that
+your program takes in the 'Program' @τ@ monad. This is especially common if you
+are processing command-line options. In that case, you may find it useful to
+initialize the program at type 'None', say, and then change to the 'Program'
+@υ@ monad you intend to run through the actual program with once the full
+settings object is available. You can do that using this function.
+
+For example:
+
+@
+main :: 'IO' ()
+main = do
+    context <- 'Core.Program.Execute.configure' \"1.0\" 'None' ('simpleConfig' ...)
+    'Core.Program.Execute.executeWith' context program1
+
+program1 :: 'Program' 'None' ()
+program1 = do
+    -- do things to form top-level application state
+    let settings =
+            Settings
+                { ...
+                }
+
+    'changeProgram' settings program2
+
+program2 :: 'Program' Settings ()
+program2 = do
+    -- now carry on with application logic
+    ...
+@
+
+This allows your code do do 'queryOptionValue' and the like in @program1@ and
+then, once all the settings and initialization is complete, you can switch to
+the actual type you intend to run at in @program2@.
+
+@since 0.6.3
+-}
+changeProgram :: υ -> Program υ α -> Program τ α
+changeProgram user' program = do
+    context1 <- ask
+    liftIO $ do
+        u <- newMVar user'
+        let context2 = context1 {applicationDataFrom = u}
+        subProgram context2 program
+
 {- |
 Write the supplied @Bytes@ to the given @Handle@. Note that in contrast to
 'write' we don't output a trailing newline.
@@ -616,7 +683,7 @@ Keep in mind that this isn't invoking a shell; arguments and their values have
 to be enumerated separately:
 
 @
-    'execProcess' [\"\/usr\/bin\/ssh\", \"-l\", \"admin\", \"203.0.113.42\", \"\\\'remote command here\\\'\"]
+    'readProcess' [\"\/usr\/bin\/ssh\", \"-l\", \"admin\", \"203.0.113.42\", \"\\\'remote command here\\\'\"]
 @
 
 having to write out the individual options and arguments and deal with
@@ -627,17 +694,19 @@ and its entire @stderr@, if any. Note that this is not a streaming interface,
 so if you're doing something that returns huge amounts of output you'll want
 to use something like __io-streams__ instead.
 
-(this wraps __typed-process__'s 'readProcess')
+(this wraps __typed-process__'s 'System.Process.Typed.readProcess')
+
+@since 0.6.4
 -}
-execProcess :: [Rope] -> Program τ (ExitCode, Rope, Rope)
-execProcess [] = error "No command provided"
-execProcess (cmd : args) =
+readProcess :: [Rope] -> Program τ (ExitCode, Rope, Rope)
+readProcess [] = error "No command provided"
+readProcess (cmd : args) =
     let cmd' = fromRope cmd
         args' = fmap fromRope args
-        task = proc cmd' args'
-        task1 = setStdin nullStream task
+        task = Typed.proc cmd' args'
+        task1 = Typed.setStdin Typed.nullStream task
         command = mconcat (List.intersperse (singletonRope ' ') (cmd : args))
-     in do
+    in  do
             debug "command" command
 
             probe <- liftIO $ do
@@ -647,10 +716,69 @@ execProcess (cmd : args) =
                     Safe.throw (CommandNotFound cmd)
                 Just _ -> do
                     (exit, out, err) <- liftIO $ do
-                        readProcess task1
+                        Typed.readProcess task1
 
                     pure (exit, intoRope out, intoRope err)
 
+execProcess :: [Rope] -> Program τ (ExitCode, Rope, Rope)
+execProcess = readProcess
+{-# DEPRECATED execProcess "Use readProcess intead" #-}
+
+{- |
+Execute a new external binary, replacing this Haskell program in memory and
+running the new binary in this program's place. The PID of the process does
+not change.
+
+This function does not return.
+
+As with 'readProcess' above, each of the arguments to the new process
+must be supplied as individual values in the list. The first argument is the
+name of the binary to be executed. The @PATH@ will be searched for the binary
+if an absolute path is not given; an exception will be thrown if it is not
+found.
+
+(this wraps __unix__'s 'executeFile' machinery, which results in an
+/execvp(3)/ standard library function call)
+
+@since 0.6.4
+-}
+execProcess_ :: [Rope] -> Program τ ()
+execProcess_ [] = error "No command provided"
+execProcess_ (cmd : args) = do
+    context <- ask
+    let cmd' = fromRope cmd
+    let args' = fmap fromRope args
+    let command = mconcat (List.intersperse (singletonRope ' ') (cmd : args))
+    let vo = outputSemaphoreFrom context
+    let out = outputChannelFrom context
+    let vl = telemetrySemaphoreFrom context
+    let tel = telemetryChannelFrom context
+
+    debug "command" command
+
+    probe <- liftIO $ do
+        findExecutable cmd'
+    case probe of
+        Nothing -> do
+            Safe.throw (CommandNotFound cmd)
+        Just _ -> do
+            liftIO $ do
+                atomically $ do
+                    writeTQueue tel Nothing
+                    writeTQueue out Nothing
+
+                _ <- forkIO $ do
+                    threadDelay 10000000
+                    putStrLn "error: Timeout"
+                    Safe.throw (ExitFailure 97)
+
+                readMVar vl
+                readMVar vo
+
+                -- does not return
+                _ <- Posix.executeFile cmd' True args' Nothing
+                pure ()
+
 {- |
 Reset the start time (used to calculate durations shown in event- and
 debug-level logging) held in the @Context@ to zero. This is useful if you want
@@ -698,7 +826,7 @@ example, to delay a second and a half, do:
 sleepThread :: Rational -> Program τ ()
 sleepThread seconds =
     let us = floor (toRational (seconds * 1e6))
-     in liftIO $ threadDelay us
+    in  liftIO $ threadDelay us
 
 {- |
 Retrieve the values of parameters parsed from options and arguments supplied
@@ -824,13 +952,17 @@ lookupOptionValue name params =
 
 data QueryParameterError
     = OptionValueMissing LongName
-    | UnableParseValue LongName
+    | UnableParseOption LongName
+    | EnvironmentVariableMissing LongName
+    | UnableParseVariable LongName
     deriving (Show)
 
 instance Exception QueryParameterError where
     displayException e = case e of
         OptionValueMissing (LongName name) -> "Option --" ++ name ++ " specified but without a value."
-        UnableParseValue (LongName name) -> "Unable to parse the value supplied to --" ++ name ++ "."
+        UnableParseOption (LongName name) -> "Unable to parse the value supplied to --" ++ name ++ "."
+        EnvironmentVariableMissing (LongName name) -> "Variable " ++ name ++ " requested but is unset."
+        UnableParseVariable (LongName name) -> "Unable to parse the value present in " ++ name ++ "."
 
 {- |
 Look to see if the user supplied a valued option and if so, what its value
@@ -869,7 +1001,7 @@ queryOptionValue' name = do
         Just parameter -> case parameter of
             Empty -> throw (OptionValueMissing name)
             Value value -> case parseExternal (packRope value) of
-                Nothing -> throw (UnableParseValue name)
+                Nothing -> throw (UnableParseOption name)
                 Just actual -> pure (Just actual)
 
 {- |
@@ -915,6 +1047,32 @@ queryEnvironmentValue name = do
             Empty -> pure Nothing
             Value str -> pure (Just (intoRope str))
 
+{- |
+Look to see if the user supplied the named environment variable and if so,
+return what its value was.
+
+Like 'queryOptionValue'' above, this function attempts to parse the supplied
+value as 'Just' the inferred type. This makes the assumption that the
+requested environment variable is populated. If it is not set in the
+environment, or is set to the empty string, then this function will return
+'Nothing'.
+
+If the attempt to parse the supplied value fails an exception will be thrown.
+
+@since 0.6.2
+-}
+queryEnvironmentValue' :: Externalize ξ => LongName -> Program τ (Maybe ξ)
+queryEnvironmentValue' name = do
+    context <- ask
+    let params = commandLineFrom context
+    case lookupKeyValue name (environmentValuesFrom params) of
+        Nothing -> error "Attempted lookup of unconfigured environment variable"
+        Just param -> case param of
+            Empty -> pure Nothing
+            Value value -> case parseExternal (packRope value) of
+                Nothing -> throw (UnableParseVariable name)
+                Just actual -> pure (Just actual)
+
 lookupEnvironmentValue :: LongName -> Parameters -> Maybe String
 lookupEnvironmentValue name params =
     case lookupKeyValue name (environmentValuesFrom params) of
